-- *******************************************************
-- KOHA 3.0 MARC 21 STANDARD DEFAULT AUTHORITY FRAMEWORKS
--
--                  PRETEST VERSION 0.0.9
--                       2007-09-29
--
--                         edited
--                   by thd for LibLime
--
--                       BASED UPON
--
--   KOHA MARC 21 STANDARD DEFAULT AUTHORITY FRAMEWORKS
--
--                  PRETEST VERSION 0.0.6
--                       2007-09-29
--
--  original default requiring greater user customisation
--               created by a few Koha Hands
--                 guided by Paul POULAIN
--
--       revised and greatly enlarged to completion,
--            well not quite complete yet today
--        but close enough for someone to have use,
--                   by thd for LibLime
-- *******************************************************


--SET CONSTRAINTS ALL DEFERRED;


-- ******************************************************
-- KOHA DEFAULT MARC 21 AUTHORITY TYPE.
-- ******************************************************

-- This authority type includes all fields and subfieds used by any
-- authority type.

INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
('', 'Default', '', '');


-- ******************************************************
-- KOHA SUPPORTED STANDARD MARC 21 AUTHORITY TYPES.
-- ******************************************************

-- These authority types are supported for for guiding the cataloguer to
-- fill authorised values in Koha MARC bibliographic editor.


INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
('PERSO_NAME', 'Personal Name', '100', 'Personal Names'),
('CORPO_NAME', 'Corporate Name', '110', 'Corporate Names'),
('MEETI_NAME', 'Meeting Name', '111', 'Meeting Name'),
('UNIF_TITLE', 'Uniform Title', '130', 'Uniform Title'),
('CHRON_TERM', 'Chronological Term', '148', 'Chronological Term'),
('TOPIC_TERM', 'Topical Term', '150', 'Topical Term'),
('GEOGR_NAME', 'Geographic Name', '151', 'Geographic Name'),
('GENRE/FORM', 'Genre/Form Term', '155', 'Genre/Form Term');


-- ******************************************************
-- KOHA UNSUPPORTED STANDARD MARC 21 AUTHORITY TYPES.
-- ******************************************************

-- These authority types are only supported for guiding the cataloguer to
-- fill authorised values in the Koha MARC bibliographic editor to the
-- extent that they have already been included in a primary authority type,
-- therefore, they have not yet been specified.
--
-- Minimal primary authorities including subdivisions may currently be
-- built by a script which uses the values in bibliographic records but
-- include no tracings and references which are necessarily not present
-- in bibliographic records.


-- INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
-- ('TOPIC_SUBD', 'General Topical Term Subdivision', '180', 'General Topical Term Subdivision'),
-- ('GEOGR_SUBD', 'Geographic Subdivision', '181', 'Geographic Subdivision'),
-- ('CHRON_SUBD', 'Chronological Subdivision', '182', 'Chronological Subdivision'),
-- ('FORM_SUBDI', 'Form Subdivision', '185', 'Form Subdivision');



-- *****************************************************************
-- MARC 21 AUTHORITY FIELDS/SUBFIELDS AND COMMMONLY USED EXTENSIONS
-- *****************************************************************


-- No support for subfield visibility is yet provided by the Koha code
-- matching the column name 'hidden' in the bibliographic framework.
--
-- Needs checking for errors but probably tolerable for use on a production.
-- A server can be upgraded easily from later versions of this file.
--
-- In the absense of more column support for qualifying the relative
-- importance of subfields to the record editor, some modest modification of
-- the default framework is needed setting the not-useful non-Koha holdings
-- subfields to not managed in Koha.

-- MARC fields including letters as part of the field identifier are from RLIN
-- and should be expected to remain along with RLIN $% subfields.  RLIN has
-- been using letters in fields because there are not enough local use number
-- fields which have not already been specified for very large union catalogue
-- networks such as RLIN itself.

-- Fields ending in a or b are temporary placeholders for information from
-- a numeric value until a non-conflicting way to treat the content under the
-- proper original numeric field is adopted.  090 for LC call numbers is much
-- too common and important so 999 is also provided as a temporary place
-- holder until all Koha code for finding control fields has been changed from
-- a numeric test of < 10 to a regular expression match of m/^00/ to prevent
-- mistaken matching of fields with letters such as 09o if they were control
-- fields.

-- Fields and subfields which belong exclusively in a classification
-- framework have been marked '[CLASSIFICATION FRAMEWORK]'.  They are retained
-- or included here becuase some had been included previously and Koha does
-- not yet have a classification framework.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('', '014', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 1, 0, NULL),
('', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('', '020', 'INTERNATIONAL STANDARD BOOK NUMBER', 'INTERNATIONAL STANDARD BOOK NUMBER', 1, 0, NULL),
('', '022', 'INTERNATIONAL STANDARD SERIAL NUMBER', 'INTERNATIONAL STANDARD SERIAL NUMBER', 1, 0, NULL),
('', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('', '031', 'MUSICAL INCIPITS INFORMATION', 'MUSICAL INCIPITS INFORMATION', 1, 0, NULL),
('', '034', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 1, 0, NULL),
('', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('', '058', 'LC CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 'LC CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 1, 0, NULL),
('', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('', '063', 'NLM CLASSIFICATION NUMBER ASSIGNED BY NLM [OBSOLETE, CAN/MARC]', 'NLM CLASSIFICATION NUMBER ASSIGNED BY NLM [OBSOLETE, CAN/MARC]', 1, 0, NULL),
('', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('', '068', 'NLM CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 'NLM CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 1, 0, NULL),
('', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('', '084', 'CLASSIFICATION SCHEME AND EDITION [CLASSIFICATION FORMAT]', 'CLASSIFICATION SCHEME AND EDITION [CLASSIFICATION FORMAT]', 0, 0, NULL),
('', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('', '088', 'DOCUMENT SHELVING NUMBER (CODOC) [OBSOLETE, CAN/MARC]', 'DOCUMENT SHELVING NUMBER (CODOC) [OBSOLETE, CAN/MARC]', 1, 0, NULL),
('', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('', '100', 'HEADING--PERSONAL NAME', 'HEADING--PERSONAL NAME', 0, 0, NULL),
('', '110', 'HEADING--CORPORATE NAME', 'HEADING--CORPORATE NAME', 0, 0, NULL),
('', '111', 'HEADING--MEETING NAME', 'HEADING--MEETING NAME', 0, 0, NULL),
('', '130', 'HEADING--UNIFORM TITLE', 'HEADING--UNIFORM TITLE', 0, 0, NULL),
('', '140', 'UNIFORM TITLE [OBSOLETE, CAN/MARC]', 'UNIFORM TITLE [OBSOLETE, CAN/MARC]', 1, 0, NULL),
('', '143', 'COLLECTIVE TITLE [OBSOLETE, CAN/MARC]', 'COLLECTIVE TITLE [OBSOLETE, CAN/MARC]', 1, 0, NULL),
('', '148', 'HEADING--CHRONOLOGICAL TERM', 'HEADING--CHRONOLOGICAL TERM', 0, 0, NULL),
('', '150', 'HEADING--TOPICAL TERM', 'HEADING--TOPICAL TERM', 0, 0, NULL),
('', '151', 'HEADING--GEOGRAPHIC NAME', 'HEADING--GEOGRAPHIC NAME', 0, 0, NULL),
('', '153', 'CLASSIFICATION NUMBER [CLASSIFICATION FORMAT]', 'CLASSIFICATION NUMBER [CLASSIFICATION FORMAT]', 0, 0, NULL),
('', '154', 'GENERAL EXPLANATORY INDEX TERM [CLASSIFICATION FORMAT]', 'GENERAL EXPLANATORY INDEX TERM [CLASSIFICATION FORMAT]', 0, 0, NULL),
('', '155', 'HEADING--GENRE/FORM TERM', 'HEADING--GENRE/FORM TERM', 0, 0, NULL),
('', '180', 'HEADING--GENERAL SUBDIVISION', 'HEADING--GENERAL SUBDIVISION', 0, 0, NULL),
('', '181', 'HEADING--GEOGRAPHIC SUBDIVISION', 'HEADING--GEOGRAPHIC SUBDIVISION', 0, 0, NULL),
('', '182', 'HEADING--CHRONOLOGICAL SUBDIVISION', 'HEADING--CHRONOLOGICAL SUBDIVISION', 0, 0, NULL),
('', '185', 'HEADING--FORM SUBDIVISION', 'HEADING--FORM SUBDIVISION', 0, 0, NULL),
('', '253', 'COMPLEX SEE REFERENCE [CLASSIFICATION FORMAT]', 'COMPLEX SEE REFERENCE [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('', '353', 'COMPLEX SEE ALSO REFERENCE [CLASSIFICATION FORMAT]', 'COMPLEX SEE ALSO REFERENCE [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('', '453', 'INVALID NUMBER TRACING [CLASSIFICATION FORMAT]', 'INVALID NUMBER TRACING [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('', '553', 'VALID NUMBER TRACING [CLASSIFICATION FORMAT]', 'VALID NUMBER TRACING [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('', '640', 'SERIES DATES OF PUBLICATION AND/OR SEQUENTIAL DESIGNATION', 'SERIES DATES OF PUBLICATION AND/OR SEQUENTIAL DESIGNATION', 1, 0, NULL),
('', '641', 'SERIES NUMBERING PECULIARITIES', 'SERIES NUMBERING PECULIARITIES', 1, 0, NULL),
('', '642', 'SERIES NUMBERING EXAMPLE', 'SERIES NUMBERING EXAMPLE', 1, 0, NULL),
('', '643', 'SERIES PLACE AND PUBLISHER/ISSUING BODY', 'SERIES PLACE AND PUBLISHER/ISSUING BODY', 1, 0, NULL),
('', '644', 'SERIES ANALYSIS PRACTICE', 'SERIES ANALYSIS PRACTICE', 1, 0, NULL),
('', '645', 'SERIES TRACING PRACTICE', 'SERIES TRACING PRACTICE', 1, 0, NULL),
('', '646', 'SERIES CLASSIFICATION PRACTICE', 'SERIES CLASSIFICATION PRACTICE', 1, 0, NULL),
('', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('', '668', 'CHARACTERS IN NONROMAN ALPHABETS [OBSOLETE]', 'CHARACTERS IN NONROMAN ALPHABETS [OBSOLETE]', 1, 0, NULL),
('', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('', '671', 'NOTE--WORK CATALOGUED [OBSOLETE, CAN/MARC ONLY]', 'NOTE--WORK CATALOGUED [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
('', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('', '676', 'NOTE--CATALOGUING RULES (NAMES/TITLES) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--CATALOGUING RULES (NAMES/TITLES) [OBSOLETE, CAN/MARC ONLY]', 0, 0, NULL),
('', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('', '684', 'AUXILIARY INSTRUCTION NOTE [CLASSIFICATION FORMAT]', 'AUXILIARY INSTRUCTION NOTE [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '685', 'NOTE--SOURCE DATA FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--SOURCE DATA FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
('', '68a', 'HISTORY NOTE [CLASSIFICATION FORMAT]', 'HISTORY NOTE [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '686', 'NOTE--SOURCE DATA NOT FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--SOURCE DATA NOT FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
('', '68b', 'RELATIONSHIP TO SOURCE NOTE [CLASSIFICATION FORMAT]', 'RELATIONSHIP TO SOURCE NOTE [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '687', 'NOTE--USAGE (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--USAGE (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
('', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('', '700', 'ESTABLISHED HEADING LINKING ENTRY--PERSONAL NAME', 'ESTABLISHED HEADING LINKING ENTRY--PERSONAL NAME', 1, 0, NULL),
('', '710', 'ESTABLISHED HEADING LINKING ENTRY--CORPORATE NAME', 'ESTABLISHED HEADING LINKING ENTRY--CORPORATE NAME', 1, 0, NULL),
('', '711', 'ESTABLISHED HEADING LINKING ENTRY--MEETING NAME', 'ESTABLISHED HEADING LINKING ENTRY--MEETING NAME', 1, 0, NULL),
('', '730', 'ESTABLISHED HEADING LINKING ENTRY--UNIFORM TITLE', 'ESTABLISHED HEADING LINKING ENTRY--UNIFORM TITLE', 1, 0, NULL),
('', '748', 'ESTABLISHED HEADING LINKING ENTRY--CHRONOLOGICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--CHRONOLOGICAL TERM', 1, 0, NULL),
('', '750', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 1, 0, NULL),
('', '751', 'ESTABLISHED HEADING LINKING ENTRY--GEOGRAPHIC NAME', 'ESTABLISHED HEADING LINKING ENTRY--GEOGRAPHIC NAME', 1, 0, NULL),
('', '753', 'INDEX TERM--UNCONTROLLED [CLASSIFICATION FORMAT]', 'INDEX TERM--UNCONTROLLED [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '754', 'INDEX TERM--FACETED TOPICAL TERMS [CLASSIFICATION FORMAT]', 'INDEX TERM--FACETED TOPICAL TERMS [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '755', 'ESTABLISHED HEADING LINKING ENTRY--GENRE/FORM TERM', 'ESTABLISHED HEADING LINKING ENTRY--GENRE/FORM TERM', 1, 0, NULL),
('', '761', 'ADD OR DIVIDE LIKE INSTRUCTIONS [CLASSIFICATION FORMAT]', 'ADD OR DIVIDE LIKE INSTRUCTIONS [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '762', 'TABLE IDENTIFICATION [CLASSIFICATION FORMAT]', 'TABLE IDENTIFICATION [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '763', 'INTERNAL SUBARRANGEMENT OR ADD TABLE ENTRY [CLASSIFICATION FORMAT]', 'INTERNAL SUBARRANGEMENT OR ADD TABLE ENTRY [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '764', 'RULE IDENTIFICATION [CLASSIFICATION FORMAT]', 'RULE IDENTIFICATION [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '765', 'SYNTHESIZED NUMBER COMPONENTS [CLASSIFICATION FORMAT]', 'SYNTHESIZED NUMBER COMPONENTS [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '766', 'SECONDARY TABLE INFORMATION [CLASSIFICATION FORMAT]', 'SECONDARY TABLE INFORMATION [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '768', 'CITATION AND PREFERENCE ORDER INSTRUCTIONS [CLASSIFICATION FORMAT]', 'CITATION AND PREFERENCE ORDER INSTRUCTIONS [CLASSIFICATION FORMAT]', 1, 0, NULL),
('', '780', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 1, 0, NULL),
('', '781', 'SUBDIVISION LINKING ENTRY--GEOGRAPHIC SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('', '782', 'SUBDIVISION LINKING ENTRY--CHRONOLOGICAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('', '785', 'SUBDIVISION LINKING ENTRY--FORM SUBDIVISION', 'SUBDIVISION LINKING ENTRY--FORM SUBDIVISION', 1, 0, NULL),
('', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);



INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '014', 'a', 'Control number of related bibliographic record', 'Control number of related bibliographic record', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '014', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '014', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '020', 'a', 'International Standard Book Number', 'International Standard Book Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '020', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '020', 'z', 'Canceled/invalid ISBN', 'Canceled/invalid ISBN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '020', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '020', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '022', 'a', 'International Standard Serial Number', 'International Standard Serial Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '022', 'y', 'Incorrect ISSN', 'Incorrect ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '022', 'z', 'Canceled ISSN', 'Canceled ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '022', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '022', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'a', 'Number of work', 'Number of work', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'b', 'Number of movement', 'Number of movement', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'c', 'Number of excerpt', 'Number of excerpt', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'd', 'Caption or heading', 'Caption or heading', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'e', 'Role', 'Role', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'g', 'Clef', 'Clef', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'm', 'Voice/instrument', 'Voice/instrument', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'n', 'Key signature', 'Key signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'o', 'Time signature', 'Time signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'p', 'Musical notation', 'Musical notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'q', 'General note', 'General note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'r', 'Key or mode', 'Key or mode', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 's', 'Coded validity note', 'Coded validity note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 't', 'Text incipit', 'Text incipit', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', '2', 'System code', 'System code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '031', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'd', 'Coordinates--westernmost longitude', 'Coordinates--westernmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'e', 'Coordinates--easternmost longitude', 'Coordinates--easternmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'f', 'Coordinates--northernmost latitude', 'Coordinates--northernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'g', 'Coordinates--southernmost latitude', 'Coordinates--southernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'j', 'Declination--northern limit', 'Declination--northern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'k', 'Declination--southern limit', 'Declination--southern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'm', 'Right ascension--eastern limit', 'Right ascension--eastern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'n', 'Right ascension--western limit', 'Right ascension--western limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'p', 'Equinox', 'Equinox', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'r', 'Distance from earth', 'Distance from earth',  1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 's', 'G-ring latitude', 'G-ring latitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 't', 'G-ring longitude', 'G-ring longitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'x', 'Beginning date', 'Beginning date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'y', 'Ending date', 'Ending date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', 'z', 'Name of extraterrestrial body', 'Name of extraterrestrial body', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '034', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '058', 'a', 'LC classification number--Single number or beginning number of a range', 'LC classification number--Single number or beginning number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '058', 'b', 'LC classification number--End number of a range', 'LC classification number--End number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '058', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '058', '5', 'Library to which class number applies', 'Library to which class number applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '058', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '063', 'a', 'NLM classification number--Single number or beginning number of a range', 'NLM classification number--Single number or beginning number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '063', 'b', 'NLM classification number--End number of a range', 'NLM classification number--End number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '063', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '068', 'a', 'NLM classification number--Single number or beginning number of a range', 'NLM classification number--Single number or beginning number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '068', 'b', 'NLM classification number--End number of a range', 'NLM classification number--End number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '068', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '068', '5', 'Library to which class number applies', 'Library to which class number applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'a', 'Classification scheme code', 'Classification scheme code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'b', 'Edition title', 'Edition title', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'c', 'Edition identifier', 'Edition identifier', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'd', 'Source edition identifier', 'Source edition identifier', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'e', 'Language code', 'Language code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'f', 'Authorization', 'Authorization', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', 'n', 'Variations', 'Variations', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '084', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '088', 'a', 'Document shelving number (CODOC)', 'Document shelving number (CODOC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'a', 'Personal name', 'Personal name', 0, 0, 1, NULL, NULL, '''100b'',''100c'',''100q'',''100d'',''100t'',''100o'',''100m'',''100r'',''100s'',''100k'',''100n'',''100p'',''100g'',''100l'',''100f'',''100h'',''100x'',''100z'',''100y'',''100v''', 0, 0, 0, '', ''),
('', '100', 'b', 'Numeration', 'Numeration', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'e', 'Relator term', 'Relator term', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '100', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 1, NULL, NULL, '''110b'',''110c'',''110d'',''110t'',''110o'',''110m'',''110r'',''110s'',''110k'',''110n'',''110p'',''110g'',''110l'',''110f'',''110h'',''110x'',''110z'',''110y'',''110v''', 0, 0, 0, '', ''),
('', '110', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'e', 'Relator term', 'Relator term', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '110', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 1, NULL, NULL, '''111e'',''111c'',''111d'',''111t'',''111s'',''111k'',''111n'',''111p'',''111g'',''111l'',''111f'',''111h'',''111x'',''111z'',''111y'',''111v''', 0, 0, 0, '', ''),
('', '111', 'b', 'Number [OBSOLETE]', 'Number [OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'j', 'Relator term', 'Relator term', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '111', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'a', 'Uniform title', 'Uniform title', 0, 0, 1, NULL, NULL, '''130o'',''130m'',''130r'',''130s'',''130d'',''130k'',''130n'',''130p'',''130g'',''130l'',''130f'',''130h'',''130t'',''130x'',''130z'',''130y'',''130v''', 0, 0, 0, '', ''),
('', '130', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '130', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'a', 'Uniform title', 'Uniform title', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'd', 'Date of treaty', 'Date of treaty', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'h', 'General material designation', 'General material designation', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'l', 'Language', 'Language', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'n', 'Number of part or section/serial, thematic, or opus number', 'Number of part or section/serial, thematic, or opus number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'o', 'Arranged or arr. for music', 'Arranged or arr. for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'p', 'Part or section', 'Part or section', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'x', 'General subject subdivision', 'General subject subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'y', 'Period subject subdivision', 'Period subject subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '140', 'z', 'Place subject subdivision', 'Place subject subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'a', 'Collective title', 'Collective title', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'd', 'Date of treaty', 'Date of treaty', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'h', 'General material designation', 'General material designation', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'l', 'Language', 'Language', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'n', 'Number of part or section/serial, thematic, or opus number', 'Number of part or section/serial, thematic, or opus number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'o', 'Arranged or arr. for music', 'Arranged or arr. for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'p', 'Part or section', 'Part or section', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '143', 's', 'Version', 'Version', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '148', 'a', 'Chronological term', 'Chronological term', 0, 0, 1, NULL, NULL, '''148y'',''148x'',''148z'',''148v''', 0, 0, 0, '', ''),
('', '148', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '148', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '148', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '148', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '148', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '148', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', 'a', 'Topical term or geographic name as entry element', 'Topical term or geographic name as entry element', 0, 0, 1, NULL, NULL, '''150x'',''150z'',''150y'',''150v''', 0, 0, 0, '', ''),
('', '150', 'b', 'Topical term following geographic name as entry element', 'Topical term following geographic name as entry element', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '150', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', 'a', 'Geographic name', 'Geographic name', 0, 0, 1, NULL, NULL, '''151z'',''151x'',''151y'',''151v''', 0, 0, 0, '', ''),
('', '151', 'b', 'Name following place as an entry element {OBSOLETE]', 'Name following place as an entry element {OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '151', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', 'j', 'Caption', 'Caption', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', 'z', 'Table identification', 'Table identification', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '153', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '154', 'a', 'General explanatory index term', 'General explanatory index term', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '154', 'b', 'General explanatory index term--Succeeding level', 'General explanatory index term--Succeeding level', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '154', 'f', 'Schedule identification', 'Schedule identification', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '154', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '154', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '155', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 1, NULL, NULL, '''155v'',''155x'',''155z'',''155y''', 0, 0, 0, '', ''),
('', '155', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '155', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '155', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '155', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '155', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '155', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '180', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '180', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '180', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '180', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '180', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '180', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '181', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '181', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '181', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '181', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '181', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '181', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '182', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '182', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '182', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '182', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '182', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '182', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '185', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '185', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '185', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '185', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '185', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '185', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', 'a', 'Classification number referred to--Single number or beginning number of span', 'Classification number referred to--Single number or beginning number of span', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', 'c', 'Classification number referred to--Ending number of span', 'Classification number referred to--Ending number of span', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', 'z', 'Table identification', 'Table identification', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '253', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', 'a', 'Classification number referred to--Single number or beginning number of span', 'Classification number referred to--Single number or beginning number of span', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', 'c', 'Classification number referred to--Ending number of span', 'Classification number referred to--Ending number of span', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', 'z', 'Table identification', 'Table identification', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '353', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'c', 'Classification element--Ending number of span', 'Classification element--Ending number of span', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'j', 'Caption', 'Caption', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 't', 'Topic', 'Topic', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', 'z', 'Table identification', 'Table identification', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '453', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'j', 'Caption', 'Caption', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 't', 'Topic', 'Topic', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', 'z', 'Table identification', 'Table identification', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '553', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '640', 'a', 'Dates of publication and/or sequential designation', 'Dates of publication and/or sequential designation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '640', 'z', 'Source of information', 'Source of information', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '640', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '640', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '641', 'a', 'Numbering peculiarities note', 'Numbering peculiarities note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '641', 'z', 'Source of information', 'Source of information', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '641', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '641', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '642', 'a', 'Series numbering example', 'Series numbering example', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '642', 'd', 'Volumes/dates to which series numbering example applies', 'Volumes/dates to which series numbering example applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '642', '5', 'Institution/copy to which field applies', 'Institution/copy to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '642', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '642', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '643', 'a', 'Place', 'Place', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '643', 'b', 'Publisher/issuing body', 'Publisher/issuing body', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '643', 'd', 'Volumes/dates to which place and publisher/issuing body apply', 'Volumes/dates to which place and publisher/issuing body apply', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '643', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '643', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '644', 'a', 'Series analysis practice', 'Series analysis practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '644', 'b', 'Exceptions to analysis practice', 'Exceptions to analysis practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '644', 'd', 'Volumes/dates to which analysis practice applies', 'Volumes/dates to which analysis practice applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '644', '5', 'Institution/copy to which field applies', 'Institution/copy to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '644', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '644', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '645', 'a', 'Series tracing practice', 'Series tracing practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '645', 'd', 'Volumes/dates to which tracing practice applies', 'Volumes/dates to which tracing practice applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '645', '5', 'Institution/copy to which field applies', 'Institution/copy to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '645', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '645', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '646', 'a', 'Series classification practice', 'Series classification practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '646', 'd', 'Volumes/dates to which classification practice applies', 'Volumes/dates to which classification practice applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '646', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '646', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '646', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '668', 'a', 'Characters in nonroman alphabet', 'Characters in nonroman alphabet', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '671', 'a', 'Citation', 'Citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '671', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '676', 'a', 'Rule number(s) and additional information', 'Rule number(s) and additional information', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '676', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', 'c', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '681', 'c', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 'Classification number--Ending number of span [CLASSIFICATION FORMAT]', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', 'j', 'Caption', 'Caption', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '684', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '685', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '685', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'a', 'New number--Single number or beginning number of span', 'New number--Single number or beginning number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'b', 'Previous number--Single number or beginning number of span', 'Previous number--Single number or beginning number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'c', 'Classification number-ending number of span', 'Classification number-ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'd', 'Date of implementation of authoritative agency', 'Date of implementation of authoritative agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'e', 'Local implementation date', 'Local implementation date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'f', 'Title and publication date', 'Title and publication date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 't', 'Topic', 'Topic', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', 'z', 'Table identification', 'Table identification', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', '2', 'Edition number', 'Edition number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68a', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '686', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '686', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 'a', 'Number in edition described in field 084--Single number or beginning number of span', 'Number in edition described in field 084--Single number or beginning number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 'b', 'Number in primary source edition--Single number or beginning number of span', 'Number in primary source edition--Single number or beginning number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 'c', 'Ending number of span', 'Ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 'o', 'Number where instructions are found--Single number or beginning number of span', 'Number where instructions are found--Single number or beginning number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 't', 'Topic', 'Topic', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', 'z', 'Table identification', 'Table identification', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', '2', 'Edition number', 'Edition number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '68b', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '687', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '687', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'a', 'Personal name', 'Personal name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'b', 'Numeration', 'Numeration', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'e', 'Relator term', 'Relator term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'k', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'r', 'Key for music', 'Key for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '3', 'Materials specified [CLASSIFICATION FORMAT]', 'Materials specified [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '4', 'Relator code', 'Relator code', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '700', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'e', 'Relator term', 'Relator term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'k', 'Form subheading', 'Form subheading', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'r', 'Key for music', 'Key for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '3', 'Materials specified [CLASSIFICATION FORMAT]', 'Materials specified [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '4', 'Relator code', 'Relator code', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '5', 'Institution to which field applies [CLASSIFICATION FORMAT]', 'Institution to which field applies [CLASSIFICATION FORMAT]', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '710', '8', 'Field link and field number', 'Field link and field number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'd', 'Date of meeting or treating signing', 'Date of meeting or treating signing', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'k', 'Form subheading', 'Form subheading', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '3', 'Materials specified', 'Materials specified', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '4', 'Relator code', 'Relator code', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '5', 'Institution to which field applies [CLASSIFICATION FORMAT]', 'Institution to which field applies [CLASSIFICATION FORMAT]', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '711', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'a', 'Uniform title', 'Uniform title', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'k', 'Form subheading', 'Form subheading', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'o', 'Arranged statement for music', 'Arranged statement for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'r', 'Key for music', 'Key for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', '3', 'Materials specified [CLASSIFICATION FORMAT]', 'Materials specified [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '730', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', 'a', 'Chronological term', 'Chronological term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', '3', 'Materials specified [CLASSIFICATION FORMAT]', 'Materials specified [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '748', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'a', 'Topical term or geographic name as entry element', 'Topical term or geographic name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'b', 'Topical term following geographic name as entry element', 'Topical term following geographic name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'c', 'Location of event', 'Location of event', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'd', 'Active date', 'Active date', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'i', 'Explanatory text [CLASSIFICATION FORMAT]', 'Explanatory text [CLASSIFICATION FORMAT]', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', '3', 'Materials specified [CLASSIFICATION FORMAT]', 'Materials specified [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '750', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'a', 'Geographic name', 'Geographic name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', '3', 'Materials specified [CLASSIFICATION FORMAT]', 'Materials specified [CLASSIFICATION FORMAT]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '751', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'a', 'Index term', 'Index term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'b', 'Index term--Succeeding level', 'Index term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'd', 'Index term referred from', 'Index term referred from', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 's', 'See also reference term', 'See also reference term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 't', 'See also reference term--Succeeding level', 'See also reference term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'u', 'Use reference term', 'Use reference term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', 'v', 'Use reference term--Succeeding level', 'Use reference term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '753', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', 'a', 'Focus term', 'Focus term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', 'b', 'Non-focus term', 'Non-focus term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', 'c', 'Facet/hierarchy designation', 'Facet/hierarchy designation', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', '3', 'Material specified', 'Material specified', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '754', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', 'a', 'Genre/form term as entry element', 'Genre/form term as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '755', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'a', 'Number where instructions are found--Single number or beginnning number of span', 'Number where instructions are found--Single number or beginnning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'b', 'Base number', 'Base number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'd', 'Divided like number', 'Divided like number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'f', 'Facet designator', 'Facet designator', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'n', 'Negative example class number', 'Negative example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'r', 'Root number', 'Root number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'x', 'Other classification number', 'Other classification number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', 'z', 'Table identification', 'Table identification', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '761', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '762', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '762', 'z', 'Table number', 'Table number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '762', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '762', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'b', 'Base number', 'Base number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'd', 'Divided like number', 'Divided like number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'j', 'Caption', 'Caption', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'm', 'Manual note', 'Manual note', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'n', 'Number where instructions are found', 'Number where instructions are found', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'r', 'Root number', 'Root number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 's', 'See reference', 'See reference', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'x', 'Other classification number', 'Other classification number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', 'z', 'Table identification', 'Table identification', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '763', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '764', 'a', 'Rule number', 'Rule number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '764', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '764', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '764', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'a', 'Number where instructions are found--Single number or beginning number of span', 'Number where instructions are found--Single number or beginning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'b', 'Base number', 'Base number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'c', 'Number where instructions are found--Ending number of span', 'Number where instructions are found--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'f', 'Facet designator', 'Facet designator', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'r', 'Root number', 'Root number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 's', 'Digits added from classification number in schedule or external table', 'Digits added from classification number in schedule or external table', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 't', 'Digits added from internal subarrangement or add table', 'Digits added from internal subarrangement or add table', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'u', 'Number being analyzed', 'Number being analyzed', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'v', 'Number in internal subarrangement or add table where instructions are found', 'Number in internal subarrangement or add table where instructions are found', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'w', 'Table identification--Internal subarrangement or add table', 'Table identification--Internal subarrangement or add table', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', 'z', 'Table identification', 'Table identification', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '765', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '766', 'a', 'Secondary table of applicability', 'Secondary table of applicability', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '766', 'y', 'Type of division', 'Type of division', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '766', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '766', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'j', 'Caption', 'Caption', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'n', 'Negative example class number', 'Negative example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 't', 'Topic used as example of citation and preference order instruction', 'Topic used as example of citation and preference order instruction', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'x', 'Exception to table of preference', 'Exception to table of preference', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', 'z', 'Table identification--Table number', 'Table identification--Table number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '768', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '780', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '781', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '782', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '785', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- PERSONAL NAME AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('PERSO_NAME', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('PERSO_NAME', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('PERSO_NAME', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('PERSO_NAME', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('PERSO_NAME', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('PERSO_NAME', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('PERSO_NAME', '014', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 1, 0, NULL),
('PERSO_NAME', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('PERSO_NAME', '020', 'INTERNATIONAL STANDARD BOOK NUMBER', 'INTERNATIONAL STANDARD BOOK NUMBER', 1, 0, NULL),
('PERSO_NAME', '022', 'INTERNATIONAL STANDARD SERIAL NUMBER', 'INTERNATIONAL STANDARD SERIAL NUMBER', 1, 0, NULL),
('PERSO_NAME', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('PERSO_NAME', '031', 'MUSICAL INCIPITS INFORMATION', 'MUSICAL INCIPITS INFORMATION', 1, 0, NULL),
('PERSO_NAME', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('PERSO_NAME', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('PERSO_NAME', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('PERSO_NAME', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('PERSO_NAME', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('PERSO_NAME', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('PERSO_NAME', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('PERSO_NAME', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('PERSO_NAME', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('PERSO_NAME', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('PERSO_NAME', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('PERSO_NAME', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('PERSO_NAME', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('PERSO_NAME', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('PERSO_NAME', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('PERSO_NAME', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('PERSO_NAME', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('PERSO_NAME', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('PERSO_NAME', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('PERSO_NAME', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('PERSO_NAME', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('PERSO_NAME', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('PERSO_NAME', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('PERSO_NAME', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('PERSO_NAME', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('PERSO_NAME', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('PERSO_NAME', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('PERSO_NAME', '100', 'HEADING--PERSONAL NAME', 'HEADING--PERSONAL NAME', 0, 0, NULL),
('PERSO_NAME', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('PERSO_NAME', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('PERSO_NAME', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('PERSO_NAME', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('PERSO_NAME', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('PERSO_NAME', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('PERSO_NAME', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('PERSO_NAME', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('PERSO_NAME', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('PERSO_NAME', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('PERSO_NAME', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('PERSO_NAME', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('PERSO_NAME', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('PERSO_NAME', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('PERSO_NAME', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('PERSO_NAME', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('PERSO_NAME', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('PERSO_NAME', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('PERSO_NAME', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('PERSO_NAME', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('PERSO_NAME', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('PERSO_NAME', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('PERSO_NAME', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('PERSO_NAME', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('PERSO_NAME', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('PERSO_NAME', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('PERSO_NAME', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('PERSO_NAME', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('PERSO_NAME', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('PERSO_NAME', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('PERSO_NAME', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('PERSO_NAME', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('PERSO_NAME', '700', 'ESTABLISHED HEADING LINKING ENTRY--PERSONAL NAME', 'ESTABLISHED HEADING LINKING ENTRY--PERSONAL NAME', 1, 0, NULL),
('PERSO_NAME', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('PERSO_NAME', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('PERSO_NAME', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('PERSO_NAME', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '014', 'a', 'Control number of related bibliographic record', 'Control number of related bibliographic record', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '014', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '014', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '020', 'a', 'International Standard Book Number', 'International Standard Book Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '020', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '020', 'z', 'Canceled/invalid ISBN', 'Canceled/invalid ISBN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '020', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '020', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '022', 'a', 'International Standard Serial Number', 'International Standard Serial Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '022', 'y', 'Incorrect ISSN', 'Incorrect ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '022', 'z', 'Canceled ISSN', 'Canceled ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '022', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '022', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'a', 'Number of work', 'Number of work', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'b', 'Number of movement', 'Number of movement', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'c', 'Number of excerpt', 'Number of excerpt', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'd', 'Caption or heading', 'Caption or heading', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'e', 'Role', 'Role', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'g', 'Clef', 'Clef', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'm', 'Voice/instrument', 'Voice/instrument', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'n', 'Key signature', 'Key signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'o', 'Time signature', 'Time signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'p', 'Musical notation', 'Musical notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'q', 'General note', 'General note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'r', 'Key or mode', 'Key or mode', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 's', 'Coded validity note', 'Coded validity note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 't', 'Text incipit', 'Text incipit', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', '2', 'System code', 'System code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '031', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'a', 'Personal name', 'Personal name', 0, 0, 1, NULL, NULL, '''100b'',''100c'',''100q'',''100d'',''100t'',''100o'',''100m'',''100r'',''100s'',''100k'',''100n'',''100p'',''100g'',''100l'',''100f'',''100h'',''100x'',''100z'',''100y'',''100v''', 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'b', 'Numeration', 'Numeration', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'e', 'Relator term', 'Relator term', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '100', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'a', 'Personal name', 'Personal name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'b', 'Numeration', 'Numeration', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'e', 'Relator term', 'Relator term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'k', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'r', 'Key for music', 'Key for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', '4', 'Relator code', 'Relator code', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '700', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '780', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('PERSO_NAME', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- CORPORATE NAME AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('CORPO_NAME', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('CORPO_NAME', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('CORPO_NAME', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('CORPO_NAME', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('CORPO_NAME', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('CORPO_NAME', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('CORPO_NAME', '014', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 1, 0, NULL),
('CORPO_NAME', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('CORPO_NAME', '020', 'INTERNATIONAL STANDARD BOOK NUMBER', 'INTERNATIONAL STANDARD BOOK NUMBER', 1, 0, NULL),
('CORPO_NAME', '022', 'INTERNATIONAL STANDARD SERIAL NUMBER', 'INTERNATIONAL STANDARD SERIAL NUMBER', 1, 0, NULL),
('CORPO_NAME', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('CORPO_NAME', '031', 'MUSICAL INCIPITS INFORMATION', 'MUSICAL INCIPITS INFORMATION', 1, 0, NULL),
('CORPO_NAME', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('CORPO_NAME', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('CORPO_NAME', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('CORPO_NAME', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('CORPO_NAME', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('CORPO_NAME', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('CORPO_NAME', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('CORPO_NAME', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('CORPO_NAME', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('CORPO_NAME', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('CORPO_NAME', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('CORPO_NAME', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('CORPO_NAME', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('CORPO_NAME', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('CORPO_NAME', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('CORPO_NAME', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('CORPO_NAME', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('CORPO_NAME', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('CORPO_NAME', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('CORPO_NAME', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('CORPO_NAME', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('CORPO_NAME', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('CORPO_NAME', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('CORPO_NAME', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('CORPO_NAME', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('CORPO_NAME', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('CORPO_NAME', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('CORPO_NAME', '110', 'HEADING--CORPORATE NAME', 'HEADING--CORPORATE NAME', 0, 0, NULL),
('CORPO_NAME', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('CORPO_NAME', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('CORPO_NAME', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('CORPO_NAME', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('CORPO_NAME', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('CORPO_NAME', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('CORPO_NAME', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('CORPO_NAME', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('CORPO_NAME', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('CORPO_NAME', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('CORPO_NAME', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('CORPO_NAME', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('CORPO_NAME', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('CORPO_NAME', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('CORPO_NAME', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('CORPO_NAME', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('CORPO_NAME', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('CORPO_NAME', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('CORPO_NAME', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('CORPO_NAME', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('CORPO_NAME', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('CORPO_NAME', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('CORPO_NAME', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('CORPO_NAME', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('CORPO_NAME', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('CORPO_NAME', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('CORPO_NAME', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('CORPO_NAME', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('CORPO_NAME', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('CORPO_NAME', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('CORPO_NAME', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('CORPO_NAME', '700', 'ESTABLISHED HEADING LINKING ENTRY--PERSONAL NAME', 'ESTABLISHED HEADING LINKING ENTRY--PERSONAL NAME', 1, 0, NULL),
('CORPO_NAME', '710', 'ESTABLISHED HEADING LINKING ENTRY--CORPORATE NAME', 'ESTABLISHED HEADING LINKING ENTRY--CORPORATE NAME', 1, 0, NULL),
('CORPO_NAME', '711', 'ESTABLISHED HEADING LINKING ENTRY--MEETING NAME', 'ESTABLISHED HEADING LINKING ENTRY--MEETING NAME', 1, 0, NULL),
('CORPO_NAME', '730', 'ESTABLISHED HEADING LINKING ENTRY--UNIFORM TITLE', 'ESTABLISHED HEADING LINKING ENTRY--UNIFORM TITLE', 1, 0, NULL),
('CORPO_NAME', '748', 'ESTABLISHED HEADING LINKING ENTRY--CHRONOLOGICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--CHRONOLOGICAL TERM', 1, 0, NULL),
('CORPO_NAME', '750', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 1, 0, NULL),
('CORPO_NAME', '751', 'ESTABLISHED HEADING LINKING ENTRY--GEOGRAPHIC NAME', 'ESTABLISHED HEADING LINKING ENTRY--GEOGRAPHIC NAME', 1, 0, NULL),
('CORPO_NAME', '755', 'ESTABLISHED HEADING LINKING ENTRY--GENRE/FORM TERM', 'ESTABLISHED HEADING LINKING ENTRY--GENRE/FORM TERM', 1, 0, NULL),
('CORPO_NAME', '780', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '781', 'SUBDIVISION LINKING ENTRY--GEOGRAPHIC SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '782', 'SUBDIVISION LINKING ENTRY--CHRONOLOGICAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '785', 'SUBDIVISION LINKING ENTRY--FORM SUBDIVISION', 'SUBDIVISION LINKING ENTRY--FORM SUBDIVISION', 1, 0, NULL),
('CORPO_NAME', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('CORPO_NAME', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('CORPO_NAME', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('CORPO_NAME', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '014', 'a', 'Control number of related bibliographic record', 'Control number of related bibliographic record', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '014', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '014', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '020', 'a', 'International Standard Book Number', 'International Standard Book Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '020', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '020', 'z', 'Canceled/invalid ISBN', 'Canceled/invalid ISBN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '020', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '020', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '022', 'a', 'International Standard Serial Number', 'International Standard Serial Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '022', 'y', 'Incorrect ISSN', 'Incorrect ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '022', 'z', 'Canceled ISSN', 'Canceled ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '022', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '022', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'a', 'Number of work', 'Number of work', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'b', 'Number of movement', 'Number of movement', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'c', 'Number of excerpt', 'Number of excerpt', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'd', 'Caption or heading', 'Caption or heading', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'e', 'Role', 'Role', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'g', 'Clef', 'Clef', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'm', 'Voice/instrument', 'Voice/instrument', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'n', 'Key signature', 'Key signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'o', 'Time signature', 'Time signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'p', 'Musical notation', 'Musical notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'q', 'General note', 'General note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'r', 'Key or mode', 'Key or mode', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 's', 'Coded validity note', 'Coded validity note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 't', 'Text incipit', 'Text incipit', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', '2', 'System code', 'System code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '031', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 1, NULL, NULL, '''110b'',''110c'',''110d'',''110t'',''110o'',''110m'',''110r'',''110s'',''110k'',''110n'',''110p'',''110g'',''110l'',''110f'',''110h'',''110x'',''110z'',''110y'',''110v''', 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'e', 'Relator term', 'Relator term', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '110', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'e', 'Relator term', 'Relator term', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'k', 'Form subheading', 'Form subheading', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'r', 'Key for music', 'Key for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', '4', 'Relator code', 'Relator code', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '710', '8', 'Field link and field number', 'Field link and field number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CORPO_NAME', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- MEETING NAME AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('MEETI_NAME', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('MEETI_NAME', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('MEETI_NAME', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('MEETI_NAME', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('MEETI_NAME', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('MEETI_NAME', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('MEETI_NAME', '014', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 1, 0, NULL),
('MEETI_NAME', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('MEETI_NAME', '020', 'INTERNATIONAL STANDARD BOOK NUMBER', 'INTERNATIONAL STANDARD BOOK NUMBER', 1, 0, NULL),
('MEETI_NAME', '022', 'INTERNATIONAL STANDARD SERIAL NUMBER', 'INTERNATIONAL STANDARD SERIAL NUMBER', 1, 0, NULL),
('MEETI_NAME', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('MEETI_NAME', '031', 'MUSICAL INCIPITS INFORMATION', 'MUSICAL INCIPITS INFORMATION', 1, 0, NULL),
('MEETI_NAME', '034', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 1, 0, NULL),
('MEETI_NAME', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('MEETI_NAME', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('MEETI_NAME', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('MEETI_NAME', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('MEETI_NAME', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('MEETI_NAME', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('MEETI_NAME', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('MEETI_NAME', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('MEETI_NAME', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('MEETI_NAME', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('MEETI_NAME', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('MEETI_NAME', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('MEETI_NAME', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('MEETI_NAME', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('MEETI_NAME', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('MEETI_NAME', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('MEETI_NAME', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('MEETI_NAME', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('MEETI_NAME', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('MEETI_NAME', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('MEETI_NAME', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('MEETI_NAME', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('MEETI_NAME', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('MEETI_NAME', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('MEETI_NAME', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('MEETI_NAME', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('MEETI_NAME', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('MEETI_NAME', '111', 'HEADING--MEETING NAME', 'HEADING--MEETING NAME', 0, 0, NULL),
('MEETI_NAME', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('MEETI_NAME', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('MEETI_NAME', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('MEETI_NAME', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('MEETI_NAME', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('MEETI_NAME', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('MEETI_NAME', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('MEETI_NAME', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('MEETI_NAME', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('MEETI_NAME', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('MEETI_NAME', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('MEETI_NAME', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('MEETI_NAME', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('MEETI_NAME', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('MEETI_NAME', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('MEETI_NAME', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('MEETI_NAME', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('MEETI_NAME', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('MEETI_NAME', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('MEETI_NAME', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('MEETI_NAME', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('MEETI_NAME', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('MEETI_NAME', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('MEETI_NAME', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('MEETI_NAME', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('MEETI_NAME', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('MEETI_NAME', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('MEETI_NAME', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('MEETI_NAME', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('MEETI_NAME', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('MEETI_NAME', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('MEETI_NAME', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('MEETI_NAME', '711', 'ESTABLISHED HEADING LINKING ENTRY--MEETING NAME', 'ESTABLISHED HEADING LINKING ENTRY--MEETING NAME', 1, 0, NULL),
('MEETI_NAME', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('MEETI_NAME', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('MEETI_NAME', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('MEETI_NAME', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '014', 'a', 'Control number of related bibliographic record', 'Control number of related bibliographic record', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '014', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '014', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '020', 'a', 'International Standard Book Number', 'International Standard Book Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '020', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '020', 'z', 'Canceled/invalid ISBN', 'Canceled/invalid ISBN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '020', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '020', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '022', 'a', 'International Standard Serial Number', 'International Standard Serial Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '022', 'y', 'Incorrect ISSN', 'Incorrect ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '022', 'z', 'Canceled ISSN', 'Canceled ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '022', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '022', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'a', 'Number of work', 'Number of work', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'b', 'Number of movement', 'Number of movement', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'c', 'Number of excerpt', 'Number of excerpt', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'd', 'Caption or heading', 'Caption or heading', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'e', 'Role', 'Role', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'g', 'Clef', 'Clef', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'm', 'Voice/instrument', 'Voice/instrument', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'n', 'Key signature', 'Key signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'o', 'Time signature', 'Time signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'p', 'Musical notation', 'Musical notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'q', 'General note', 'General note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'r', 'Key or mode', 'Key or mode', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 's', 'Coded validity note', 'Coded validity note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 't', 'Text incipit', 'Text incipit', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', '2', 'System code', 'System code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '031', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'd', 'Coordinates--westernmost longitude', 'Coordinates--westernmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'e', 'Coordinates--easternmost longitude', 'Coordinates--easternmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'f', 'Coordinates--northernmost latitude', 'Coordinates--northernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'g', 'Coordinates--southernmost latitude', 'Coordinates--southernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'j', 'Declination--northern limit', 'Declination--northern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'k', 'Declination--southern limit', 'Declination--southern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'm', 'Right ascension--eastern limit', 'Right ascension--eastern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'n', 'Right ascension--western limit', 'Right ascension--western limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'p', 'Equinox', 'Equinox', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'r', 'Distance from earth', 'Distance from earth', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 's', 'G-ring latitude', 'G-ring latitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 't', 'G-ring longitude', 'G-ring longitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'x', 'Beginning date', 'Beginning date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'y', 'Ending date', 'Ending date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', 'z', 'Name of extraterrestrial body', 'Name of extraterrestrial body', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '034', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 1, NULL, NULL, '''111e'',''111c'',''111d'',''111t'',''111s'',''111k'',''111n'',''111p'',''111g'',''111l'',''111f'',''111h'',''111x'',''111z'',''111y'',''111v''', 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'b', 'Number [OBSOLETE]', 'Number [OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'j', 'Relator term', 'Relator term', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '111', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'd', 'Date of meeting or treating signing', 'Date of meeting or treating signing', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'k', 'Form subheading', 'Form subheading', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', '3', 'Materials specified', 'Materials specified', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', '4', 'Relator code', 'Relator code', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '711', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('MEETI_NAME', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- UNIFORM TITLE AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('UNIF_TITLE', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('UNIF_TITLE', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('UNIF_TITLE', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('UNIF_TITLE', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('UNIF_TITLE', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('UNIF_TITLE', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('UNIF_TITLE', '014', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 'LINK TO BIBLIOGRAPHIC RECORD FOR SERIAL OR MULTIPART ITEM', 1, 0, NULL),
('UNIF_TITLE', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '020', 'INTERNATIONAL STANDARD BOOK NUMBER', 'INTERNATIONAL STANDARD BOOK NUMBER', 1, 0, NULL),
('UNIF_TITLE', '022', 'INTERNATIONAL STANDARD SERIAL NUMBER', 'INTERNATIONAL STANDARD SERIAL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('UNIF_TITLE', '031', 'MUSICAL INCIPITS INFORMATION', 'MUSICAL INCIPITS INFORMATION', 1, 0, NULL),
('UNIF_TITLE', '034', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 1, 0, NULL),
('UNIF_TITLE', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('UNIF_TITLE', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('UNIF_TITLE', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('UNIF_TITLE', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('UNIF_TITLE', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('UNIF_TITLE', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('UNIF_TITLE', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('UNIF_TITLE', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('UNIF_TITLE', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('UNIF_TITLE', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('UNIF_TITLE', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('UNIF_TITLE', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('UNIF_TITLE', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('UNIF_TITLE', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('UNIF_TITLE', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('UNIF_TITLE', '130', 'HEADING--UNIFORM TITLE', 'HEADING--UNIFORM TITLE', 0, 0, NULL),
('UNIF_TITLE', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('UNIF_TITLE', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('UNIF_TITLE', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('UNIF_TITLE', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('UNIF_TITLE', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('UNIF_TITLE', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('UNIF_TITLE', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('UNIF_TITLE', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('UNIF_TITLE', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('UNIF_TITLE', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('UNIF_TITLE', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('UNIF_TITLE', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('UNIF_TITLE', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('UNIF_TITLE', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('UNIF_TITLE', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('UNIF_TITLE', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('UNIF_TITLE', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('UNIF_TITLE', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('UNIF_TITLE', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('UNIF_TITLE', '640', 'SERIES DATES OF PUBLICATION AND/OR SEQUENTIAL DESIGNATION', 'SERIES DATES OF PUBLICATION AND/OR SEQUENTIAL DESIGNATION', 1, 0, NULL),
('UNIF_TITLE', '641', 'SERIES NUMBERING PECULIARITIES', 'SERIES NUMBERING PECULIARITIES', 1, 0, NULL),
('UNIF_TITLE', '642', 'SERIES NUMBERING EXAMPLE', 'SERIES NUMBERING EXAMPLE', 1, 0, NULL),
('UNIF_TITLE', '643', 'SERIES PLACE AND PUBLISHER/ISSUING BODY', 'SERIES PLACE AND PUBLISHER/ISSUING BODY', 1, 0, NULL),
('UNIF_TITLE', '644', 'SERIES ANALYSIS PRACTICE', 'SERIES ANALYSIS PRACTICE', 1, 0, NULL),
('UNIF_TITLE', '645', 'SERIES TRACING PRACTICE', 'SERIES TRACING PRACTICE', 1, 0, NULL),
('UNIF_TITLE', '646', 'SERIES CLASSIFICATION PRACTICE', 'SERIES CLASSIFICATION PRACTICE', 1, 0, NULL),
('UNIF_TITLE', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('UNIF_TITLE', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('UNIF_TITLE', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('UNIF_TITLE', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('UNIF_TITLE', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('UNIF_TITLE', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('UNIF_TITLE', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('UNIF_TITLE', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('UNIF_TITLE', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('UNIF_TITLE', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('UNIF_TITLE', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('UNIF_TITLE', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('UNIF_TITLE', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('UNIF_TITLE', '730', 'ESTABLISHED HEADING LINKING ENTRY--UNIFORM TITLE', 'ESTABLISHED HEADING LINKING ENTRY--UNIFORM TITLE', 1, 0, NULL),
('UNIF_TITLE', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('UNIF_TITLE', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('UNIF_TITLE', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('UNIF_TITLE', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '014', 'a', 'Control number of related bibliographic record', 'Control number of related bibliographic record', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '014', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '014', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '020', 'a', 'International Standard Book Number', 'International Standard Book Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '020', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '020', 'z', 'Canceled/invalid ISBN', 'Canceled/invalid ISBN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '020', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '020', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '022', 'a', 'International Standard Serial Number', 'International Standard Serial Number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '022', 'y', 'Incorrect ISSN', 'Incorrect ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '022', 'z', 'Canceled ISSN', 'Canceled ISSN', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '022', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '022', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'a', 'Number of work', 'Number of work', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'b', 'Number of movement', 'Number of movement', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'c', 'Number of excerpt', 'Number of excerpt', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'd', 'Caption or heading', 'Caption or heading', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'e', 'Role', 'Role', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'g', 'Clef', 'Clef', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'm', 'Voice/instrument', 'Voice/instrument', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'n', 'Key signature', 'Key signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'o', 'Time signature', 'Time signature', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'p', 'Musical notation', 'Musical notation', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'q', 'General note', 'General note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'r', 'Key or mode', 'Key or mode', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 's', 'Coded validity note', 'Coded validity note', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 't', 'Text incipit', 'Text incipit', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', '2', 'System code', 'System code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '031', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'd', 'Coordinates--westernmost longitude', 'Coordinates--westernmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'e', 'Coordinates--easternmost longitude', 'Coordinates--easternmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'f', 'Coordinates--northernmost latitude', 'Coordinates--northernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'g', 'Coordinates--southernmost latitude', 'Coordinates--southernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'j', 'Declination--northern limit', 'Declination--northern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'k', 'Declination--southern limit', 'Declination--southern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'm', 'Right ascension--eastern limit', 'Right ascension--eastern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'n', 'Right ascension--western limit', 'Right ascension--western limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'p', 'Equinox', 'Equinox', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'r', 'Distance from earth', 'Distance from earth', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 's', 'G-ring latitude', 'G-ring latitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 't', 'G-ring longitude', 'G-ring longitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'x', 'Beginning date', 'Beginning date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'y', 'Ending date', 'Ending date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', 'z', 'Name of extraterrestrial body', 'Name of extraterrestrial body', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '034', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'a', 'Uniform title', 'Uniform title', 0, 0, 1, NULL, NULL, '''130o'',''130m'',''130r'',''130s'',''130d'',''130k'',''130n'',''130p'',''130g'',''130l'',''130f'',''130h'',''130t'',''130x'',''130z'',''130y'',''130v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'f', 'Date of a work', 'Date of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'h', 'Medium', 'Medium', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'k', 'Form subheading', 'Form subheading', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'l', 'Language of a work', 'Language of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'r', 'Key for music', 'Key for music', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 's', 'Version', 'Version', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 't', 'Title of a work', 'Title of a work', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '130', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '640', 'a', 'Dates of publication and/or sequential designation', 'Dates of publication and/or sequential designation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '640', 'z', 'Source of information', 'Source of information', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '640', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '640', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '641', 'a', 'Numbering peculiarities note', 'Numbering peculiarities note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '641', 'z', 'Source of information', 'Source of information', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '641', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '641', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '642', 'a', 'Series numbering example', 'Series numbering example', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '642', 'd', 'Volumes/dates to which series numbering example applies', 'Volumes/dates to which series numbering example applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '642', '5', 'Institution/copy to which field applies', 'Institution/copy to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '642', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '642', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '643', 'a', 'Place', 'Place', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '643', 'b', 'Publisher/issuing body', 'Publisher/issuing body', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '643', 'd', 'Volumes/dates to which place and publisher/issuing body apply', 'Volumes/dates to which place and publisher/issuing body apply', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '643', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '643', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '644', 'a', 'Series analysis practice', 'Series analysis practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '644', 'b', 'Exceptions to analysis practice', 'Exceptions to analysis practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '644', 'd', 'Volumes/dates to which analysis practice applies', 'Volumes/dates to which analysis practice applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '644', '5', 'Institution/copy to which field applies', 'Institution/copy to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '644', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '644', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '645', 'a', 'Series tracing practice', 'Series tracing practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '645', 'd', 'Volumes/dates to which tracing practice applies', 'Volumes/dates to which tracing practice applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '645', '5', 'Institution/copy to which field applies', 'Institution/copy to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '645', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '645', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '646', 'a', 'Series classification practice', 'Series classification practice', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '646', 'd', 'Volumes/dates to which classification practice applies', 'Volumes/dates to which classification practice applies', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '646', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '646', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '646', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'a', 'Uniform title', 'Uniform title', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'f', 'Date of a work', 'Date of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'h', 'Medium', 'Medium', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'k', 'Form subheading', 'Form subheading', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'l', 'Language of a work', 'Language of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'o', 'Arranged statement for music', 'Arranged statement for music', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'r', 'Key for music', 'Key for music', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 's', 'Version', 'Version', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 't', 'Title of a work', 'Title of a work', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '730', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('UNIF_TITLE', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- TOPICAL TERM AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('CHRON_TERM', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('CHRON_TERM', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('CHRON_TERM', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('CHRON_TERM', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('CHRON_TERM', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('CHRON_TERM', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('CHRON_TERM', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('CHRON_TERM', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('CHRON_TERM', '034', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 1, 0, NULL),
('CHRON_TERM', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('CHRON_TERM', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('CHRON_TERM', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('CHRON_TERM', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('CHRON_TERM', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('CHRON_TERM', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('CHRON_TERM', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('CHRON_TERM', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('CHRON_TERM', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('CHRON_TERM', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('CHRON_TERM', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('CHRON_TERM', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('CHRON_TERM', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('CHRON_TERM', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('CHRON_TERM', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('CHRON_TERM', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('CHRON_TERM', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('CHRON_TERM', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('CHRON_TERM', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('CHRON_TERM', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('CHRON_TERM', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('CHRON_TERM', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('CHRON_TERM', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('CHRON_TERM', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('CHRON_TERM', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('CHRON_TERM', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('CHRON_TERM', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('CHRON_TERM', '148', 'HEADING--CHRONOLOGICAL TERM', 'HEADING--CHRONOLOGICAL TERM', 0, 0, NULL),
('CHRON_TERM', '182', 'HEADING--CHRONOLOGICAL SUBDIVISION', 'HEADING--CHRONOLOGICAL SUBDIVISION', 0, 0, NULL),
('CHRON_TERM', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('CHRON_TERM', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('CHRON_TERM', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('CHRON_TERM', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('CHRON_TERM', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('CHRON_TERM', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('CHRON_TERM', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('CHRON_TERM', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('CHRON_TERM', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('CHRON_TERM', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('CHRON_TERM', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('CHRON_TERM', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('CHRON_TERM', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('CHRON_TERM', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('CHRON_TERM', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('CHRON_TERM', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('CHRON_TERM', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('CHRON_TERM', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('CHRON_TERM', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('CHRON_TERM', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('CHRON_TERM', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('CHRON_TERM', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('CHRON_TERM', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('CHRON_TERM', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('CHRON_TERM', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('CHRON_TERM', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('CHRON_TERM', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('CHRON_TERM', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('CHRON_TERM', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('CHRON_TERM', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('CHRON_TERM', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('CHRON_TERM', '748', 'ESTABLISHED HEADING LINKING ENTRY--CHRONOLOGICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--CHRONOLOGICAL TERM', 1, 0, NULL),
('CHRON_TERM', '782', 'SUBDIVISION LINKING ENTRY--CHRONOLOGICAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('CHRON_TERM', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('CHRON_TERM', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('CHRON_TERM', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('CHRON_TERM', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'd', 'Coordinates--westernmost longitude', 'Coordinates--westernmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'e', 'Coordinates--easternmost longitude', 'Coordinates--easternmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'f', 'Coordinates--northernmost latitude', 'Coordinates--northernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'g', 'Coordinates--southernmost latitude', 'Coordinates--southernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'j', 'Declination--northern limit', 'Declination--northern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'k', 'Declination--southern limit', 'Declination--southern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'm', 'Right ascension--eastern limit', 'Right ascension--eastern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'n', 'Right ascension--western limit', 'Right ascension--western limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'p', 'Equinox', 'Equinox', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'r', 'Distance from earth', 'Distance from earth', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 's', 'G-ring latitude', 'G-ring latitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 't', 'G-ring longitude', 'G-ring longitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'x', 'Beginning date', 'Beginning date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'y', 'Ending date', 'Ending date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', 'z', 'Name of extraterrestrial body', 'Name of extraterrestrial body', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '034', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '148', 'a', 'Chronological term', 'Chronological term', 0, 0, 1, NULL, NULL, '''148y'',''148x'',''148z'',''148v''', 0, 0, 0, '', ''),
('CHRON_TERM', '148', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '148', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '148', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '148', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '148', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '148', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '182', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '182', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '182', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '182', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '182', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '182', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', 'a', 'Chronological term', 'Chronological term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '748', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '782', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('CHRON_TERM', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- TOPICAL TERM AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('TOPIC_TERM', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('TOPIC_TERM', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('TOPIC_TERM', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('TOPIC_TERM', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('TOPIC_TERM', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('TOPIC_TERM', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('TOPIC_TERM', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('TOPIC_TERM', '034', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 1, 0, NULL),
('TOPIC_TERM', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('TOPIC_TERM', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('TOPIC_TERM', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('TOPIC_TERM', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('TOPIC_TERM', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('TOPIC_TERM', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('TOPIC_TERM', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('TOPIC_TERM', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('TOPIC_TERM', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('TOPIC_TERM', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('TOPIC_TERM', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('TOPIC_TERM', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('TOPIC_TERM', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('TOPIC_TERM', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('TOPIC_TERM', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('TOPIC_TERM', '150', 'HEADING--TOPICAL TERM', 'HEADING--TOPICAL TERM', 0, 0, NULL),
('TOPIC_TERM', '180', 'HEADING--GENERAL SUBDIVISION', 'HEADING--GENERAL SUBDIVISION', 0, 0, NULL),
('TOPIC_TERM', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('TOPIC_TERM', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('TOPIC_TERM', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('TOPIC_TERM', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('TOPIC_TERM', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('TOPIC_TERM', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('TOPIC_TERM', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('TOPIC_TERM', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('TOPIC_TERM', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('TOPIC_TERM', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('TOPIC_TERM', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('TOPIC_TERM', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('TOPIC_TERM', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('TOPIC_TERM', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('TOPIC_TERM', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('TOPIC_TERM', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('TOPIC_TERM', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('TOPIC_TERM', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('TOPIC_TERM', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('TOPIC_TERM', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('TOPIC_TERM', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('TOPIC_TERM', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('TOPIC_TERM', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('TOPIC_TERM', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('TOPIC_TERM', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('TOPIC_TERM', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('TOPIC_TERM', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('TOPIC_TERM', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('TOPIC_TERM', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('TOPIC_TERM', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('TOPIC_TERM', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('TOPIC_TERM', '750', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 1, 0, NULL),
('TOPIC_TERM', '780', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 1, 0, NULL),
('TOPIC_TERM', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('TOPIC_TERM', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('TOPIC_TERM', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('TOPIC_TERM', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'd', 'Coordinates--westernmost longitude', 'Coordinates--westernmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'e', 'Coordinates--easternmost longitude', 'Coordinates--easternmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'f', 'Coordinates--northernmost latitude', 'Coordinates--northernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'g', 'Coordinates--southernmost latitude', 'Coordinates--southernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'j', 'Declination--northern limit', 'Declination--northern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'k', 'Declination--southern limit', 'Declination--southern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'm', 'Right ascension--eastern limit', 'Right ascension--eastern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'n', 'Right ascension--western limit', 'Right ascension--western limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'p', 'Equinox', 'Equinox', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'r', 'Distance from earth', 'Distance from earth', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 's', 'G-ring latitude', 'G-ring latitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 't', 'G-ring longitude', 'G-ring longitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'x', 'Beginning date', 'Beginning date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'y', 'Ending date', 'Ending date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', 'z', 'Name of extraterrestrial body', 'Name of extraterrestrial body', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '034', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', 'a', 'Topical term or geographic name as entry element', 'Topical term or geographic name as entry element', 0, 0, 1, NULL, NULL, '''150x'',''150z'',''150y'',''150v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '150', 'b', 'Topical term following geographic name as entry element', 'Topical term following geographic name as entry element', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '150', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '180', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '180', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '180', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '180', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '180', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '180', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'a', 'Topical term or geographic name as entry element', 'Topical term or geographic name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'b', 'Topical term following geographic name as entry element', 'Topical term following geographic name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'c', 'Location of event', 'Location of event', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'd', 'Active date', 'Active date', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '750', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '780', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('TOPIC_TERM', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- GEOGRAPHIC NAME AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('GEOGR_NAME', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('GEOGR_NAME', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('GEOGR_NAME', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('GEOGR_NAME', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('GEOGR_NAME', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('GEOGR_NAME', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('GEOGR_NAME', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('GEOGR_NAME', '034', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 'CODED CARTOGRAPHIC MATHEMATICAL DATA', 1, 0, NULL),
('GEOGR_NAME', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('GEOGR_NAME', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('GEOGR_NAME', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('GEOGR_NAME', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('GEOGR_NAME', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('GEOGR_NAME', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('GEOGR_NAME', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('GEOGR_NAME', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('GEOGR_NAME', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('GEOGR_NAME', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('GEOGR_NAME', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('GEOGR_NAME', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('GEOGR_NAME', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('GEOGR_NAME', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('GEOGR_NAME', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('GEOGR_NAME', '151', 'HEADING--GEOGRAPHIC NAME', 'HEADING--GEOGRAPHIC NAME', 0, 0, NULL),
('GEOGR_NAME', '181', 'HEADING--GEOGRAPHIC SUBDIVISION', 'HEADING--GEOGRAPHIC SUBDIVISION', 0, 0, NULL),
('GEOGR_NAME', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('GEOGR_NAME', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('GEOGR_NAME', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('GEOGR_NAME', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('GEOGR_NAME', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('GEOGR_NAME', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('GEOGR_NAME', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('GEOGR_NAME', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('GEOGR_NAME', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('GEOGR_NAME', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('GEOGR_NAME', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('GEOGR_NAME', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('GEOGR_NAME', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('GEOGR_NAME', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('GEOGR_NAME', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('GEOGR_NAME', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('GEOGR_NAME', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('GEOGR_NAME', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('GEOGR_NAME', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('GEOGR_NAME', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('GEOGR_NAME', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('GEOGR_NAME', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('GEOGR_NAME', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('GEOGR_NAME', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('GEOGR_NAME', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('GEOGR_NAME', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('GEOGR_NAME', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('GEOGR_NAME', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('GEOGR_NAME', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('GEOGR_NAME', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('GEOGR_NAME', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('GEOGR_NAME', '751', 'ESTABLISHED HEADING LINKING ENTRY--GEOGRAPHIC NAME', 'ESTABLISHED HEADING LINKING ENTRY--GEOGRAPHIC NAME', 1, 0, NULL),
('GEOGR_NAME', '781', 'SUBDIVISION LINKING ENTRY--GEOGRAPHIC SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('GEOGR_NAME', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('GEOGR_NAME', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('GEOGR_NAME', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('GEOGR_NAME', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'd', 'Coordinates--westernmost longitude', 'Coordinates--westernmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'e', 'Coordinates--easternmost longitude', 'Coordinates--easternmost longitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'f', 'Coordinates--northernmost latitude', 'Coordinates--northernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'g', 'Coordinates--southernmost latitude', 'Coordinates--southernmost latitude', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'j', 'Declination--northern limit', 'Declination--northern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'k', 'Declination--southern limit', 'Declination--southern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'm', 'Right ascension--eastern limit', 'Right ascension--eastern limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'n', 'Right ascension--western limit', 'Right ascension--western limit', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'p', 'Equinox', 'Equinox', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'r', 'Distance from earth', 'Distance from earth', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 's', 'G-ring latitude', 'G-ring latitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 't', 'G-ring longitude', 'G-ring longitude', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'x', 'Beginning date', 'Beginning date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'y', 'Ending date', 'Ending date', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', 'z', 'Name of extraterrestrial body', 'Name of extraterrestrial body', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '034', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', 'a', 'Geographic name', 'Geographic name', 0, 0, 1, NULL, NULL, '''151z'',''151x'',''151y'',''151v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '151', 'b', 'Name following place as an entry element {OBSOLETE]', 'Name following place as an entry element {OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '151', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '181', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '181', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '181', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '181', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '181', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '181', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'a', 'Geographic name', 'Geographic name', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '751', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '781', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GEOGR_NAME', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


-- ******************************************************



-- *******************************************************************
-- GENRE/FORM TERM AUTHORITY FIELDS/SUBFIELDS.
-- *******************************************************************

-- These ought to be adjusted for different less conflicting and more
-- rationally chosen fields and subfields but I had left that for last.

-- ADJUST ME
-- Use values from your dump of auth_tag_structure and auth_subfield_structure
-- to provide support for your Koha database.


INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value) VALUES
('GENRE/FORM', '000', 'LEADER', 'LEADER', 0, 1, NULL),
('GENRE/FORM', '001', 'CONTROL NUMBER', 'CONTROL NUMBER', 0, 1, NULL),
('GENRE/FORM', '003', 'CONTROL NUMBER IDENTIFIER', 'CONTROL NUMBER IDENTIFIER', 0, 1, NULL),
('GENRE/FORM', '005', 'DATE AND TIME OF LATEST TRANSACTION', 'DATE AND TIME OF LATEST TRANSACTION', 0, 1, NULL),
('GENRE/FORM', '008', 'FIXED-LENGTH DATA ELEMENTS', 'FIXED-LENGTH DATA ELEMENTS', 0, 1, NULL),
('GENRE/FORM', '010', 'LIBRARY OF CONGRESS CONTROL NUMBER', 'LIBRARY OF CONGRESS CONTROL NUMBER', 0, 0, NULL),
('GENRE/FORM', '016', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 'NATIONAL BIBLIOGRAPHIC AGENCY CONTROL NUMBER', 1, 0, NULL),
('GENRE/FORM', '024', 'OTHER STANDARD IDENTIFIER', 'OTHER STANDARD IDENTIFIER', 1, 0, NULL),
('GENRE/FORM', '035', 'SYSTEM CONTROL NUMBER', 'SYSTEM CONTROL NUMBER', 1, 0, NULL),
('GENRE/FORM', '040', 'CATALOGING SOURCE', 'CATALOGING SOURCE', 0, 1, NULL),
('GENRE/FORM', '042', 'AUTHENTICATION CODE', 'AUTHENTICATION CODE', 0, 0, NULL),
('GENRE/FORM', '043', 'GEOGRAPHIC AREA CODE', 'GEOGRAPHIC AREA CODE', 0, 0, NULL),
('GENRE/FORM', '045', 'TIME PERIOD OF HEADING', 'TIME PERIOD OF HEADING', 0, 0, NULL),
('GENRE/FORM', '050', 'LIBRARY OF CONGRESS CALL NUMBER', 'LIBRARY OF CONGRESS CALL NUMBER', 1, 0, NULL),
('GENRE/FORM', '052', 'GEOGRAPHIC CLASSIFICATION', 'GEOGRAPHIC CLASSIFICATION', 1, 0, NULL),
('GENRE/FORM', '053', 'LC CLASSIFICATION NUMBER', 'LC CLASSIFICATION NUMBER', 1, 0, NULL),
('GENRE/FORM', '055', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 'NATIONAL LIBRARY AND ARCHIVE OF CANADA CALL NUMBER', 1, 0, NULL),
('GENRE/FORM', '060', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 'NATIONAL LIBRARY OF MEDICINE CALL NUMBER', 1, 0, NULL),
('GENRE/FORM', '065', 'OTHER CLASSIFICATION NUMBER', 'OTHER CLASSIFICATION NUMBER', 1, 0, NULL),
('GENRE/FORM', '066', 'CHARACTER SETS PRESENT', 'CHARACTER SETS PRESENT', 0, 0, NULL),
('GENRE/FORM', '070', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 'NATIONAL AGRICULTURAL LIBRARY CALL NUMBER', 1, 0, NULL),
('GENRE/FORM', '072', 'SUBJECT CATEGORY CODE', 'SUBJECT CATEGORY CODE', 1, 0, NULL),
('GENRE/FORM', '073', 'SUBDIVISION USAGE', 'SUBDIVISION USAGE', 0, 0, NULL),
('GENRE/FORM', '082', 'DEWEY DECIMAL CALL NUMBER', 'DEWEY DECIMAL CALL NUMBER', 1, 0, NULL),
('GENRE/FORM', '083', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 'DEWEY DECIMAL CLASSIFICATION NUMBER', 1, 0, NULL),
('GENRE/FORM', '086', 'GOVERNMENT DOCUMENT CALL NUMBER', 'GOVERNMENT DOCUMENT CALL NUMBER', 1, 0, NULL),
('GENRE/FORM', '087', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 'GOVERNMENT DOCUMENT CLASSIFICATION NUMBER', 1, 0, NULL),
('GENRE/FORM', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (OCLC)', 1, 0, NULL),
('GENRE/FORM', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
('GENRE/FORM', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
('GENRE/FORM', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('GENRE/FORM', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
('GENRE/FORM', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
('GENRE/FORM', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
('GENRE/FORM', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
('GENRE/FORM', '155', 'HEADING--GENRE/FORM TERM', 'HEADING--GENRE/FORM TERM', 0, 0, NULL),
('GENRE/FORM', '185', 'HEADING--FORM SUBDIVISION', 'HEADING--FORM SUBDIVISION', 0, 0, NULL),
('GENRE/FORM', '260', 'COMPLEX SEE REFERENCE--SUBJECT', 'COMPLEX SEE REFERENCE--SUBJECT', 1, 0, NULL),
('GENRE/FORM', '360', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 'COMPLEX SEE ALSO REFERENCE--SUBJECT', 1, 0, NULL),
('GENRE/FORM', '400', 'SEE FROM TRACING--PERSONAL NAME', 'SEE FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('GENRE/FORM', '410', 'SEE FROM TRACING--CORPORATE NAME', 'SEE FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('GENRE/FORM', '411', 'SEE FROM TRACING--MEETING NAME', 'SEE FROM TRACING--MEETING NAME', 1, 0, NULL),
('GENRE/FORM', '430', 'SEE FROM TRACING--UNIFORM TITLE', 'SEE FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('GENRE/FORM', '448', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 'SEE FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('GENRE/FORM', '450', 'SEE FROM TRACING--TOPICAL TERM', 'SEE FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('GENRE/FORM', '451', 'SEE FROM TRACING--GEOGRAPHIC NAME', 'SEE FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('GENRE/FORM', '455', 'SEE FROM TRACING--GENRE/FORM TERM', 'SEE FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('GENRE/FORM', '480', 'SEE FROM TRACING--GENERAL SUBDIVISION', 'SEE FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '481', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '482', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '485', 'SEE FROM TRACING--FORM SUBDIVISION', 'SEE FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '500', 'SEE ALSO FROM TRACING--PERSONAL NAME', 'SEE ALSO FROM TRACING--PERSONAL NAME', 1, 0, NULL),
('GENRE/FORM', '510', 'SEE ALSO FROM TRACING--CORPORATE NAME', 'SEE ALSO FROM TRACING--CORPORATE NAME', 1, 0, NULL),
('GENRE/FORM', '511', 'SEE ALSO FROM TRACING--MEETING NAME', 'SEE ALSO FROM TRACING--MEETING NAME', 1, 0, NULL),
('GENRE/FORM', '530', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 'SEE ALSO FROM TRACING--UNIFORM TITLE', 1, 0, NULL),
('GENRE/FORM', '548', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 'SEE ALSO FROM TRACING--CHRONOLOGICAL TERM', 1, 0, NULL),
('GENRE/FORM', '550', 'SEE ALSO FROM TRACING--TOPICAL TERM', 'SEE ALSO FROM TRACING--TOPICAL TERM', 1, 0, NULL),
('GENRE/FORM', '551', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 'SEE ALSO FROM TRACING--GEOGRAPHIC NAME', 1, 0, NULL),
('GENRE/FORM', '555', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 'SEE ALSO FROM TRACING--GENRE/FORM TERM', 1, 0, NULL),
('GENRE/FORM', '580', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 'SEE ALSO FROM TRACING--GENERAL SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '581', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 'SEE ALSO FROM TRACING--GEOGRAPHIC SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '582', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 'SEE ALSO FROM TRACING--CHRONOLOGICAL SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '585', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 'SEE ALSO FROM TRACING--FORM SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '663', 'COMPLEX SEE ALSO REFERENCE--NAME', 'COMPLEX SEE ALSO REFERENCE--NAME', 0, 0, NULL),
('GENRE/FORM', '664', 'COMPLEX SEE REFERENCE--NAME', 'COMPLEX SEE REFERENCE--NAME', 0, 0, NULL),
('GENRE/FORM', '665', 'HISTORY REFERENCE', 'HISTORY REFERENCE', 0, 0, NULL),
('GENRE/FORM', '666', 'GENERAL EXPLANATORY REFERENCE--NAME', 'GENERAL EXPLANATORY REFERENCE--NAME', 0, 0, NULL),
('GENRE/FORM', '667', 'NONPUBLIC GENERAL NOTE', 'NONPUBLIC GENERAL NOTE', 1, 0, NULL),
('GENRE/FORM', '670', 'SOURCE DATA FOUND', 'SOURCE DATA FOUND', 1, 0, NULL),
('GENRE/FORM', '675', 'SOURCE DATA NOT FOUND', 'SOURCE DATA NOT FOUND', 0, 0, NULL),
('GENRE/FORM', '678', 'BIOGRAPHICAL OR HISTORICAL DATA', 'BIOGRAPHICAL OR HISTORICAL DATA', 1, 0, NULL),
('GENRE/FORM', '680', 'PUBLIC GENERAL NOTE', 'PUBLIC GENERAL NOTE', 1, 0, NULL),
('GENRE/FORM', '681', 'SUBJECT EXAMPLE TRACING NOTE', 'SUBJECT EXAMPLE TRACING NOTE', 1, 0, NULL),
('GENRE/FORM', '682', 'DELETED HEADING INFORMATION', 'DELETED HEADING INFORMATION', 0, 0, NULL),
('GENRE/FORM', '683', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('GENRE/FORM', '688', 'APPLICATION HISTORY NOTE', 'APPLICATION HISTORY NOTE', 1, 0, NULL),
('GENRE/FORM', '750', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 'ESTABLISHED HEADING LINKING ENTRY--TOPICAL TERM', 1, 0, NULL),
('GENRE/FORM', '755', 'ESTABLISHED HEADING LINKING ENTRY--GENRE/FORM TERM', 'ESTABLISHED HEADING LINKING ENTRY--GENRE/FORM TERM', 1, 0, NULL),
('GENRE/FORM', '780', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 'SUBDIVISION LINKING ENTRY--GENERAL SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '785', 'SUBDIVISION LINKING ENTRY--FORM SUBDIVISION', 'SUBDIVISION LINKING ENTRY--FORM SUBDIVISION', 1, 0, NULL),
('GENRE/FORM', '788', 'COMPLEX LINKING ENTRY DATA', 'COMPLEX LINKING ENTRY DATA', 0, 0, NULL),
('GENRE/FORM', '856', 'ELECTRONIC LOCATION AND ACCESS', 'ELECTRONIC LOCATION AND ACCESS', 1, 0, NULL),
('GENRE/FORM', '880', 'ALTERNATE GRAPHIC REPRESENTATION', 'ALTERNATE GRAPHIC REPRESENTATION', 1, 0, NULL);


INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode) VALUES
('GENRE/FORM', '000', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '001', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '003', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '005', '@', 'control field', 'control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '008', '@', 'fixed length control field', 'fixed length control field', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '010', 'a', 'LC control number', 'LC control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '010', 'z', 'Canceled/invalid LC control number', 'Canceled/invalid LC control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '010', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '016', 'a', 'Record control number', 'Record control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '016', 'z', 'Canceled/invalid record control number', 'Canceled/invalid record control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '016', '2', 'Source', 'Source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '016', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', 'a', 'Standard number or code', 'Standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', 'c', 'Terms of availability', 'Terms of availability', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', 'd', 'Additional codes following the standard number or code', 'Additional codes following the standard number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', 'z', 'Canceled/invalid standard number or code', 'Canceled/invalid standard number or code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', '2', 'Source of number or code', 'Source of number or code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '024', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '035', 'z', 'Canceled/invalid control number', 'Canceled/invalid control number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '035', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '035', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', 'a', 'Original cataloging agency', 'Original cataloging agency', 0, 1, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', 'b', 'Language of cataloging', 'Language of cataloging', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', 'c', 'Transcribing agency', 'Transcribing agency', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', 'd', 'Modifying agency', 'Modifying agency', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', 'e', 'Description conventions', 'Description conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', 'f', 'Subject heading or thesaurus conventions', 'Subject heading or thesaurus conventions', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '040', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '042', 'a', 'Authentication code', 'Authentication code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '043', 'a', 'Geographic area code', 'Geographic area code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '043', 'b', 'Local GAC code', 'Local GAC code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '043', 'c', 'ISO code', 'ISO code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '043', '2', 'Source of local code', 'Source of local code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '043', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '043', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '045', 'a', 'Time period code', 'Time period code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '045', 'b', 'Formatted 9999 B.C. through C.E. time period', 'Formatted 9999 B.C. through C.E. time period', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '045', 'c', 'Formatted pre-9999 B.C. time period', 'Formatted pre-9999 B.C. time period', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '045', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '045', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '050', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '050', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '050', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '050', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '050', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '050', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '052', 'a', 'Geographic classification area code', 'Geographic classification area code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '052', 'b', 'Geographic classification subarea code', 'Geographic classification subarea code', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '052', 'd', 'Populated place name', 'Populated place name', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '052', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '052', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '052', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '053', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '053', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '053', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '053', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '053', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '053', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '055', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '055', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '055', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '055', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '055', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '055', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '060', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '060', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '060', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '060', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '060', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '060', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', 'a', 'Classification number element--single number or beginning of span', 'Classification number element--single number or beginning of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '065', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '066', 'a', 'Primary G0 character set', 'Primary G0 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '066', 'b', 'Primary G1 character set', 'Primary G1 character set', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '066', 'c', 'Alternate G0 or G1 character set', 'Alternate G0 or G1 character set', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '070', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '070', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '070', 'd', 'Volume/dates to which call number applies', 'Volume/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '070', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '070', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '072', 'a', 'Subject category code', 'Subject category code', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '072', 'x', 'Subject category code subdivision', 'Subject category code subdivision', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '072', '2', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '072', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '072', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '073', 'a', 'Subdivision usage', 'Subdivision usage', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '073', 'z', 'Code source', 'Code source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '073', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '073', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '082', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '083', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', 'a', 'Call number', 'Call number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '086', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '087', 'a', 'Classification number element--Single number or beginning number of span', 'Classification number element--Single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '087', 'b', 'Classification number element--Ending number of span', 'Classification number element--Ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '087', 'c', 'Explanatory information', 'Explanatory information', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '087', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '087', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '087', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', 'z', 'Cancelled/invalid call number', 'Cancelled/invalid call number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', '2', 'Number source', 'Number source', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '090', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '091', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '092', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '093', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '096', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', 'a', 'Classification number element--single number or beginning number of span', 'Classification number element--single number or beginning number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', 'b', 'Classification number element--ending number of span', 'Classification number element--ending number of span', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', 'c', 'Explanatory term', 'Explanatory term', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', 'z', 'Table identification--table number', 'Table identification--table number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '097', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '098', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', 'a', 'Classification number', 'Classification number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', 'b', 'Item number', 'Item number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', 'd', 'Volumes/dates to which call number applies', 'Volumes/dates to which call number applies', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', '2', 'Edition number', 'Edition number', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', '6', 'Linkage', 'Linkage', 0, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '099', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 0, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '155', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 1, NULL, NULL, '''155v'',''155x'',''155z'',''155y''', 0, 0, 0, '', ''),
('GENRE/FORM', '155', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '155', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '155', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '155', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '155', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '155', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '185', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '185', 'x', 'General subdivision', 'General subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '185', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '185', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '185', '6', 'Linkage', 'Linkage', 0, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '185', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 1, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '260', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '260', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '260', '6', 'Linkage', 'Linkage', 0, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '260', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 2, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '360', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '360', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '360', '6', 'Linkage', 'Linkage', 0, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '360', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 3, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'a', 'Personal name', 'Personal name', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'b', 'Numeration', 'Numeration', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '400', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'e', 'Relator term', 'Relator term', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '410', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '411', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'a', 'Uniform title', 'Uniform title', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'f', 'Date of a work', 'Date of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'h', 'Medium', 'Medium', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'k', 'Form subheading', 'Form subheading', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'l', 'Language of a work', 'Language of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'r', 'Key for music', 'Key for music', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 's', 'Version', 'Version', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 't', 'Title of a work', 'Title of a work', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '430', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'a', 'Chronological term', 'Chronological term', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '448', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '450', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'a', 'Geographic name', 'Geographic name', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '451', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '455', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '480', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '481', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '482', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', 'w', 'Control subfield', 'Control subfield', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', 'x', 'General subdivision', 'General subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', '6', 'Linkage', 'Linkage', 0, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '485', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 4, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'a', 'Personal name', 'Personal name', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'b', 'Numeration', 'Numeration', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'c', 'Titles and other words associated with a name', 'Titles and other words associated with a name', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'd', 'Dates associated with a name', 'Dates associated with a name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'j', 'Attribution qualifier', 'Attribution qualifier', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'q', 'Fuller form of name', 'Fuller form of name', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '500', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'a', 'Corporate name or jurisdiction name as entry element', 'Corporate name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'b', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'd', 'Date of meeting or treaty signing', 'Date of meeting or treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'e', 'Relator term', 'Relator term', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '510', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'a', 'Meeting name or jurisdiction name as entry element', 'Meeting name or jurisdiction name as entry element', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'c', 'Location of meeting', 'Location of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'd', 'Date of meeting', 'Date of meeting', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'e', 'Subordinate unit', 'Subordinate unit', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'n', 'Number of part/section/meeting', 'Number of part/section/meeting', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'q', 'Name of meeting following jurisdiction name entry element', 'Name of meeting following jurisdiction name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '511', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'a', 'Uniform title', 'Uniform title', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'd', 'Date of treaty signing', 'Date of treaty signing', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'f', 'Date of a work', 'Date of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'g', 'Miscellaneous information', 'Miscellaneous information', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'h', 'Medium', 'Medium', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'k', 'Form subheading', 'Form subheading', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'l', 'Language of a work', 'Language of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'm', 'Medium of performance for music', 'Medium of performance for music', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'n', 'Number of part/section of a work', 'Number of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'o', 'Arranged statement for music', 'Arranged statement for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'p', 'Name of part/section of a work', 'Name of part/section of a work', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'r', 'Key for music', 'Key for music', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 's', 'Version', 'Version', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 't', 'Title of a work', 'Title of a work', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '530', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'a', 'Chronological term', 'Chronological term', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '548', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'a', 'Topical term or geographic name entry element', 'Topical term or geographic name entry element', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'b', 'Topical term following geographic name entry element', 'Topical term following geographic name entry element', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '550', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'a', 'Geographic name', 'Geographic name', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '551', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'a', 'Genre/form term', 'Genre/form term', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '555', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '580', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '581', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '582', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', 'i', 'Reference instruction phrase', 'Reference instruction phrase', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', 'w', 'Control subfield', 'Control subfield', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', 'x', 'General subdivision', 'General subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', '0', 'Record control number', 'Record control number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', '6', 'Linkage', 'Linkage', 0, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '585', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 5, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '663', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '663', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '663', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '663', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '663', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '664', 'a', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '664', 'b', 'Heading referred to', 'Heading referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '664', 't', 'Title referred to', 'Title referred to', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '664', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '664', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '665', 'a', 'History reference', 'History reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '665', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '665', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '666', 'a', 'General explanatory reference', 'General explanatory reference', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '666', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '666', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '667', 'a', 'Nonpublic general note', 'Nonpublic general note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '667', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '667', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '667', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '670', 'a', 'Source citation', 'Source citation', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '670', 'b', 'Information found', 'Information found', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '670', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '670', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '670', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '675', 'a', 'Source citation', 'Source citation', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '675', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '675', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '678', 'a', 'Biographical or historical data', 'Biographical or historical data', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '678', 'b', 'Expansion', 'Expansion', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '678', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '678', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '678', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '680', 'a', 'Heading or subdivision term', 'Heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '680', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '680', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '680', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '680', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '680', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '681', 'a', 'Subject heading or subdivision term', 'Subject heading or subdivision term', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '681', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '681', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '681', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '681', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '682', 'a', 'Replacement heading', 'Replacement heading', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '682', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '682', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '682', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', 'c', 'Classification number--Ending number of span', 'Classification number--Ending number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', 'p', 'Corresponding classification field', 'Corresponding classification field', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', 'z', 'Table identification', 'Table identification', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '683', '8', 'Field link and sequence number', 'Field link and sequence number', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '688', 'a', 'Application history note', 'Application history note', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '688', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '688', '6', 'Linkage', 'Linkage', 0, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '688', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 6, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'a', 'Topical term or geographic name as entry element', 'Topical term or geographic name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'b', 'Topical term following geographic name as entry element', 'Topical term following geographic name as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'c', 'Location of event', 'Location of event', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'd', 'Active date', 'Active date', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '750', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', 'a', 'Genre/form term as entry element', 'Genre/form term as entry element', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '755', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '780', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', 'v', 'Form subdivision', 'Form subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', 'w', 'Control subfield', 'Control subfield', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', 'x', 'General subdivision', 'General subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', 'y', 'Chronological subdivision', 'Chronological subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', 'z', 'Geographic subdivision', 'Geographic subdivision', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '785', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', 'a', 'Heading referred to', 'Heading referred to', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', 'i', 'Explanatory text', 'Explanatory text', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', '0', 'Record control number', 'Record control number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', '2', 'Source of heading or term', 'Source of heading or term', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', '5', 'Institution to which field applies', 'Institution to which field applies', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', '6', 'Linkage', 'Linkage', 0, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '788', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 7, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'a', 'Host name', 'Host name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'b', 'Access number', 'Access number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'c', 'Compression information', 'Compression information', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'd', 'Path', 'Path', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'f', 'Electronic name', 'Electronic name', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'h', 'Processor of request', 'Processor of request', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'i', 'Instruction', 'Instruction', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'j', 'Bits per second', 'Bits per second', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'm', 'Contact for access assistance', 'Contact for access assistance', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'n', 'Name of location of host', 'Name of location of host', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'o', 'Operating system', 'Operating system', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'p', 'Port', 'Port', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'q', 'Electronic format type', 'Electronic format type', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'r', 'Settings', 'Settings', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 's', 'File size', 'File size', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 't', 'Terminal emulation', 'Terminal emulation', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'u', 'Uniform Resource Identifier', 'Uniform Resource Identifier', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'v', 'Hours access method available', 'Hours access method available', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'w', 'Record control number', 'Record control number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'x', 'Nonpublic note', 'Nonpublic note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'y', 'Link text', 'Link text', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', 'z', 'Public note', 'Public note', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', '2', 'Access method', 'Access method', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', '3', 'Materials specified', 'Materials specified', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', '6', 'Linkage', 'Linkage', 0, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '856', '8', 'Field link and sequence number', 'Field link and sequence number', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '6', '6', '6', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', ''),
('GENRE/FORM', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, 0, '', '');


